@@ -97,6 +97,7 @@ pub fn graph(
                     0,
                     Vec::new(),
                     true,
+                    &mut Vec::new(),
                 )
                 {
                     Ok(f) => (f.0, f.1, options, f.2),
@@ -293,7 +294,7 @@ pub fn graph(
             {
                 Auto
             };
-            if (options.lines || lines) && lines ^ options.lines
+            if (options.lines == crate::Auto::Auto && lines) || options.lines == crate::Auto::True
             {
                 fg.axes2d()
                     .set_x_grid(!cli)
@@ -774,7 +775,8 @@ pub fn graph(
                         &[PointSymbol(options.point_style), Color(&colors.re1col)],
                     );
             }
-            else if options.lines || lines
+            else if (options.lines == crate::Auto::Auto && lines)
+                || options.lines == crate::Auto::True
             {
                 fg.axes3d()
                     .set_x_grid(!cli)
@@ -1266,7 +1268,7 @@ pub fn get_list_2d(
         {
             matches!(
                 s.as_str(),
-                "±" | "cubic" | "quadratic" | "quartic" | "unity"
+                "±" | "cubic" | "quadratic" | "quad" | "quartic" | "unity"
             )
         }
         else
@@ -1279,7 +1281,7 @@ pub fn get_list_2d(
             {
                 matches!(
                     s.as_str(),
-                    "±" | "cubic" | "quadratic" | "quartic" | "unity"
+                    "±" | "cubic" | "quadratic" | "quad" | "quartic" | "unity"
                 )
             }
             else
@@ -1576,7 +1578,7 @@ pub fn get_list_3d(
         {
             matches!(
                 s.as_str(),
-                "±" | "cubic" | "quadratic" | "quartic" | "unity"
+                "±" | "cubic" | "quadratic" | "quad" | "quartic" | "unity"
             )
         }
         else
@@ -1589,7 +1591,7 @@ pub fn get_list_3d(
             {
                 matches!(
                     s.as_str(),
-                    "±" | "cubic" | "quadratic" | "quartic" | "unity"
+                    "±" | "cubic" | "quadratic" | "quad" | "quartic" | "unity"
                 )
             }
             else
@@ -2053,6 +2055,23 @@ fn get_data(
                 }
                 Matrix(m) =>
                 {
+                    if !m.iter().all(|a| a.len() == m[0].len())
+                    {
+                        print!(
+                            "\x1b[G\x1b[Kbad matrix data in {}\x1b[G\n{}",
+                            input,
+                            prompt(func.2, &colors)
+                        );
+                        stdout().flush().unwrap();
+                        return (
+                            (false, false),
+                            (false, false),
+                            false,
+                            true,
+                            Default::default(),
+                            Default::default(),
+                        );
+                    }
                     lines = true;
                     match m[0].len()
                     {
