@@ -25,12 +25,12 @@ FLAGS: --help (this message)\x1b[G\n\
 --ticks=[num](,[num](,[num])) sets amount of ticks, optionally set different x/y/z ticks, -2 will be auto, -1 will be at every whole number, 0 will be none\x1b[G\n\
 --onaxis toggles showing the ticks on the x/y/z axis\x1b[G\n\
 --prompt toggles the prompt\x1b[G\n\
---color toggles color output, toggled by default when running from arguments\x1b[G\n\
+--color=true/false/auto toggles color output, toggled by default when running from arguments\x1b[G\n\
 --comma toggles comma seperation\x1b[G\n\
 --graph toggles graphing\x1b[G\n\
 --vars disables default variables and kalc.vars\x1b[G\n\
 --default sets to default settings and ignores kalc.vars\x1b[G\n\
---line toggles line graphing\x1b[G\n\
+--line=true/false/auto toggles line graphing\x1b[G\n\
 --rt toggles real time printing\x1b[G\n\
 --polar toggles displaying polar vectors\x1b[G\n\
 --frac toggles fraction display\x1b[G\n\
@@ -42,7 +42,8 @@ FLAGS: --help (this message)\x1b[G\n\
 --surface displays a colored surface(based on z value) for 3d graphing, only supports 1 graph\x1b[G\n\
 --scalegraph scales the y part of a 2d graph to the users screen size, setting --windowsize=x,y makes the ratio more accurate\x1b[G\n\
 --saveto=[file] saves the graph as a png to the given file, --windowsize=x,y for resolution\x1b[G\n\
---siunits toggles keeping stuff in si units, a newton will show as 'm s^-2 kg' instead of 'N'\x1b[G\n\n\
+--siunits toggles keeping stuff in si units, a newton will show as 'm s^-2 kg' instead of 'N'\x1b[G\n\
+--keepzeros dont remove trailing zeros\x1b[G\n\n\
 - flags can be executed in runtime just without the dashes\x1b[G\n\
 - \"colors=\" to see color settings\x1b[G\n\
 - \"exit\" to exit the program\x1b[G\n\
@@ -58,7 +59,7 @@ FLAGS: --help (this message)\x1b[G\n\
 - \"...=\" display parsed input, show values of stuff like xr/deci/prec etc\x1b[G\n\
 - \"f...=null\" to delete a function or variable\x1b[G\n\
 - \"{{x,y,z...}}\" to define a cartesian vector\x1b[G\n\
-- \"[radius,theta,phi]\" to define a polar vector (same as car{{vec}})\x1b[G\n\
+- \"[r,θ,φ]\" to define a polar vector (same as car{{r,θ,φ}})\x1b[G\n\
 - \"f(x)#g(x)\" to graph multiple things\x1b[G\n\
 - \"{{vec}}#\" to graph a vector\x1b[G\n\
 - \"{{mat}}#\" to graph a matrix\x1b[G\n\
@@ -68,7 +69,7 @@ FLAGS: --help (this message)\x1b[G\n\
 - \"{{{{a,b,c}},{{d,e,f}},{{g,h,i}}}}\" to define a 3x3 matrix\x1b[G\n\
 - \"rnd\" to generate a random number\x1b[G\n\
 - \"epoch\" to get time in seconds since unix epoch\x1b[G\n\
-- Alt+Enter will not graph whatever is present\x1b[G\n\
+- Alt+Enter will not print output while still graphing/defining variables\x1b[G\n\
 - \"help {{thing}}\" to get more detail on a function/option/feature\x1b[G\n\
 - \"help help\" to list all things to query\x1b[G\n\n\
 Order of Operations:\x1b[G\n\
@@ -94,36 +95,44 @@ Functions:\x1b[G\n\
 - ceil, floor, round, int, frac\x1b[G\n\
 - fact, doublefact, subfact\x1b[G\n\
 - sinc, cis, exp\x1b[G\n\
-- zeta, eta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
+- zeta, eta, gamma, lower_gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
 - re, im, onlyreal, onlyimag, split(x+yi), next(n,to)\x1b[G\n\
 - unity(n,k) gets all solutions for x in x^k=n\x1b[G\n\
-- factors, prime, isprime, gcd, lcm\x1b[G\n\
+- factors, nth_prime, is_prime, is_nan, is_inf, is_finite, gcd, lcm\x1b[G\n\
 - slog(a,b), ssrt(k,a) (k is lambert w branch)\x1b[G\n\
 - piecewise/pw({{value,cond}},{{value2,cond2}}...) (when first condition is met from left to right. value elsewards is nan)\x1b[G\n\
 - vec(var,func,start,end) mat(var,func,start,end) (makes a vector/matrix) start..end is a shortcut to vec(n,n,start,end)\x1b[G\n\
-- tofreq{{a,b,c...}}, tolist{{{{a,b}},{{c,d}}...}}, tolist{{a,b,c}} (sorts and counts how many time each number occurs, tolist takes that kind of data and reverses it)\x1b[G\n\
-- variance/var, covariance/cov, standarddeviation/σ (sample-bias corrected)\x1b[G\n\
+- to_freq{{a,b,c...}}, to_list{{{{a,b}},{{c,d}}...}}, to_list{{a,b,c}} (sorts and counts how many time each number occurs, to_list takes that kind of data and reverses it)\x1b[G\n\
+- variance/var, covariance/cov, standarddeviation/sd/σ (sample-bias corrected), skew/skewness, kurtosis (excess)\x1b[G\n\
 - percentile({{vec}},nth) (gets number at nth percentile), percentilerank({{vec}},x) (gets percentile rank for x point), quartiles{{vec}} (gets quartiles for data set)\x1b[G\n\
-- normP(μ,σ,x) (normal distribution pdf) normD(z)/normD(x,μ,σ) (area under curve to the left of z score cdf)\x1b[G\n\
-- betaP(α,β,x) (beta distribution pdf) I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\x1b[G\n\
+- norm_pdf(x,μ,σ) (normal distribution pdf) normD(z)/norm_cdf(x,μ,σ) (area under curve to the left of z score cdf)\x1b[G\n\
+- beta_pdf(x,α,β) (beta distribution pdf) beta_cdf/I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\x1b[G\n\
+- gamma_pdf(x,k,θ), gamma_cdf(x,k,θ), lognorm_pdf(x,μ,σ), lognorm_cdf(x,μ,σ), binomial_pmf(k,n,p), binomial_cdf(k,n,p), neg_binomial_pmf(k,r,p), neg_binomial_cdf(k,r,p)\x1b[G\n\
+- geometric_pmf(k,p), geometric_cdf(k,p), poisson_pmf(x,λ), poisson_cdf(x,λ), hypergeometric_pmf(k,N,K,n), hypergeometric_cdf(k,N,K,n), neg_hypergeometric_pmf(k,N,K,r), neg_hypergeometric_cdf(k,N,K,r)\x1b[G\n\
+- rand_norm(μ,σ), rand_uniform(a,b), rand_int(a,b), rand_gamma(k,θ), rand_lognorm(μ,σ), rand_binomial(n,p), rand_neg_binomial(r,p)\x1b[G\n\
+- rand_geometric(k,p), rand_bernoulli(p), rand_poisson(λ), rand_hypergeometric(N,K,n), rand_neg_hypergeometric(N,K,r)\x1b[G\n\
 - roll{{a,b,c...}} rolls die, dice{{a,b,c...}} gets the frequency data any amount of different sided die, where a/b/c are number of faces for each die, both also accept {{{{first_dice_face,# of die}},{{second_dice_face,# of die}}...}}\x1b[G\n\
-- weighted{{{{a,n1}},{{b,n2}}..}} rolls a weighted die where a and b are face values and n1 and n2 are their weights\x1b[G\n\
+- rand_weighted{{{{a,n1}},{{b,n2}}..}} rolls a weighted die where a and b are face values and n1 and n2 are their weights\x1b[G\n\
 - An(n,k), Ap(n,t) eulerian numbers and polynomials\x1b[G\n\
 - lim(x,f(x),point (,side)) both sides are checked by default, -1 for left, 1 for right\x1b[G\n\
 - slope(x,f(x),point (,nth derivitive) (,0) ), can add a 0 to the args to not combine the x and y slopes for parametric equations, same for area\x1b[G\n\
 - area(x,f(x),from,to (,0) ), length(x,f(x),from,to), surfacearea(a,b,z(a,b),startb,endb,starta,enda)\x1b[G\n\
 - solve(x,f(x) (,point)) employs newtons method to find the root of a function at a starting point, assumes 0 if no point given, outputs Nan if newton method fails\x1b[G\n\
+- extrema(x,f(x) (,point)) employs newtons method to find the extrema of a function at a starting point, assumes 0 if no point given, outputs Nan if newton method fails, outputs {{x,y,positive/negitive concavity}}\x1b[G\n\
 - iter(x,f(x),p,n), f(x) iterated n times at point p, add \",1\" to args to show steps\x1b[G\n\n\
 Vector operations/functions:\x1b[G\n\
 - dot({{vec1}},{{vec2}}), cross({{vec1}},{{vec2}}), proj/project({{vec1}},{{vec2}})\x1b[G\n\
 - angle({{vec1}},{{vec2}})\x1b[G\n\
 - norm, normalize\x1b[G\n\
 - abs, len, any, all\x1b[G\n\
-- max, min, mean, median, mode, sort\x1b[G\n\
+- max, min, mean, median, mode, sort, geo_mean\x1b[G\n\
 - reverse, link\x1b[G\n\
 - part({{vec}},col), sum, prod\x1b[G\n\
-- convert to polar: pol{{vec}} outputs (radius, theta, phi)\x1b[G\n\
-- convert to cartesian: car{{vec}} outputs (x, y, z)\x1b[G\n\
+- pol{{x,y,z}} outputs (r, θ, φ)\x1b[G\n\
+- pol{{x,y}} outputs (r, θ)\x1b[G\n\
+- car{{r,θ,φ}} outputs (x, y, z)\x1b[G\n\
+- car{{r,θ}} outputs (x, y)\x1b[G\n\
+- cyl{{x,y,z}} outputs (p, θ, z)\x1b[G\n\
 - other functions are applied like sqrt{{2,4}}={{sqrt(2),sqrt(4)}}\x1b[G\n\n\
 Matrix operations/functions:\x1b[G\n\
 - eigenvalues, eigenvectors\x1b[G\n\
@@ -132,12 +141,13 @@ Matrix operations/functions:\x1b[G\n\
 - part({{mat}},col,row), flatten, sum, prod\x1b[G\n\
 - abs, norm\x1b[G\n\
 - len, wid\x1b[G\n\
-- max, min, mean, mode\x1b[G\n\
+- max, min, mean, mode, weighted_mean{{{{n,weight}}...}}\x1b[G\n\
 - iden(n) produces an n dimension identity matrix\x1b[G\n\
-- rotate(theta), rotate(yaw,pitch,roll) produces a rotational matrix\x1b[G\n\
+- rotate(θ), rotate(yaw,pitch,roll) produces a rotational matrix\x1b[G\n\
 - sort(mat) sorts rows by first column\x1b[G\n\
 - interpolate/inter(mat,x) using lagrange interpolation interpolates a 2xN matrix along x, matrix should be organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
-- lineofbestfit/lobf(mat,x) line of best fit for a matrix organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
+- lineofbestfit/lobf(mat,x) line of best fit for numerous 2d values, with no x values it will spit out the m/b values for line equation in form of mx+b, mat should be organized like {{{{x0,y0}},{{x1,y1}} ... {{xN,yN}}}}\x1b[G\n\
+- plane(mat,x,y) finds the plane that 3, 3d points lie on, with no x/y arg it will spit out the a/b/c values for the equation of plane in ax+by+c form, mat should be in form of {{{{x0,y0,z0}},{{x1,y1,z1}},{{x2,y2,z2}}}}\x1b[G\n\
 - other functions are applied like sqrt{{{{2,4}},{{5,6}}}}={{{{sqrt(2),sqrt(4)}},{{sqrt(5),sqrt(6)}}}}\x1b[G\n\n\
 Constants:\x1b[G\n\
 - c: speed of light, 299792458 m/s\x1b[G\n\
