@@ -15,13 +15,14 @@ mod units;
 //lib, gui
 use crate::{
     complex::NumStr,
+    functions::functions_with_args,
     graph::graph,
     help::help_for,
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
     misc::{
         clear, clearln, convert, get_terminal_dimensions, handle_err, insert_last, prompt,
-        read_single_char, write,
+        read_single_char, to_output, write,
     },
     options::{arg_opts, commands, equal_to, file_opts, silent_commands},
     parse::input_var,
@@ -37,7 +38,7 @@ use rug::Complex;
 use std::{
     env::args,
     fs::{File, OpenOptions},
-    io::{stdin, stdout, BufRead, BufReader, IsTerminal, Write},
+    io::{stdin, stdout, BufRead, BufReader, IsTerminal, Stdout, Write},
     thread::JoinHandle,
     time::Instant,
 };
@@ -135,6 +136,13 @@ pub enum AngleType
     Degrees,
     Gradians,
 }
+#[derive(Copy, Clone, PartialEq)]
+pub enum Auto
+{
+    True,
+    False,
+    Auto,
+}
 #[derive(Default, Copy, Clone, PartialEq)]
 pub struct HowGraphing
 {
@@ -178,7 +186,7 @@ pub struct Options
     frac: Fractions,
     real_time_output: bool,
     decimal_places: usize,
-    color: bool,
+    color: Auto,
     prompt: bool,
     comma: bool,
     prec: u32,
@@ -192,7 +200,7 @@ pub struct Options
     samples_2d: usize,
     samples_3d: (usize, usize),
     point_style: char,
-    lines: bool,
+    lines: Auto,
     multi: bool,
     tabbed: bool,
     allow_vars: bool,
@@ -206,6 +214,7 @@ pub struct Options
     units: bool,
     si_units: bool,
     window_size: (usize, usize),
+    keep_zeros: bool,
 }
 impl Default for Options
 {
@@ -216,36 +225,36 @@ impl Default for Options
             angle: Radians,
             graphtype: GraphType::Normal,
             base: (10, 10),
-            ticks: (20.0, 20.0, 20.0),
+            ticks: (16.0, 16.0, 16.0),
             onaxis: true,
             polar: false,
             frac: Fractions {
                 num: true,
-                vec: false,
+                vec: true,
                 mat: false,
             },
             real_time_output: true,
             decimal_places: 12,
-            color: true,
+            color: Auto::Auto,
             prompt: true,
             comma: false,
             prec: 512,
             graph_prec: 128,
-            xr: (-10.0, 10.0),
-            yr: (-10.0, 10.0),
-            zr: (-10.0, 10.0),
+            xr: (-8.0, 8.0),
+            yr: (-8.0, 8.0),
+            zr: (-8.0, 8.0),
             vxr: (0.0, 0.0),
             vyr: (0.0, 0.0),
             vzr: (0.0, 0.0),
             samples_2d: 8192,
             samples_3d: (256, 256),
             point_style: '.',
-            lines: false,
+            lines: Auto::Auto,
             multi: true,
             tabbed: false,
             allow_vars: true,
             debug: false,
-            slowcheck: 300,
+            slowcheck: 256,
             interactive: true,
             surface: false,
             scale_graph: false,
@@ -254,6 +263,7 @@ impl Default for Options
             units: true,
             si_units: false,
             window_size: (0, 0),
+            keep_zeros: false,
         }
     }
 }
@@ -310,11 +320,22 @@ fn main()
     let mut stdout = stdout();
     if options.interactive
     {
+        if options.color == Auto::Auto
+        {
+            options.color = Auto::True;
+        }
         terminal::enable_raw_mode().unwrap();
         print!(
             "\x1b[G\x1b[K{}{}",
             prompt(options, &colors),
-            if options.color { "\x1b[0m" } else { "" }
+            if options.color == Auto::True
+            {
+                "\x1b[0m"
+            }
+            else
+            {
+                ""
+            }
         );
         stdout.flush().unwrap();
     }
@@ -449,7 +470,7 @@ fn main()
                                     continue 'upper;
                                 }
                             }
-                            for (i, j) in vars.clone().iter().enumerate()
+                            for (i, j) in vars.iter().enumerate()
                             {
                                 if j.name.len() <= le.len()
                                 {
@@ -475,6 +496,10 @@ fn main()
     options.base = base;
     let (mut file, mut unmod_lines) = if options.interactive || options.stay_interactive
     {
+        if options.color == Auto::Auto
+        {
+            options.color = Auto::True;
+        }
         let file_path = &(dir.clone() + "/kalc.history");
         if File::open(file_path).is_err()
         {
@@ -492,7 +517,10 @@ fn main()
     }
     else
     {
-        options.color = !options.color;
+        if options.color == Auto::Auto
+        {
+            options.color = Auto::False;
+        }
         (None, None)
     };
     let mut handles: Vec<JoinHandle<()>> = Vec::new();
@@ -516,122 +544,118 @@ fn main()
             input = args.remove(0).chars().collect();
             let output;
             let funcvar;
-            let mut options = options;
-            let mut unparsed = input.clone();
             {
-                let split = input.split(|c| c == &';');
-                let count = split.clone().count();
-                if count != 1
+                let mut options = options;
+                let mut unparsed = input.clone();
                 {
-                    unparsed = split.clone().last().unwrap().to_vec();
-                    for (i, s) in split.enumerate()
+                    let split = input.split(|c| c == &';');
+                    let count = split.clone().count();
+                    if count != 1
                     {
-                        if i == count - 1
+                        unparsed = split.clone().last().unwrap().to_vec();
+                        for (i, s) in split.enumerate()
                         {
-                            break;
-                        }
-                        silent_commands(
-                            &mut options,
-                            &s.iter()
-                                .copied()
-                                .filter(|&c| !c.is_whitespace())
-                                .collect::<Vec<char>>(),
-                        );
-                        if s.contains(&'=')
-                        {
-                            if let Err(s) =
-                                set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
+                            if i == count - 1
                             {
-                                println!("{}", s);
-                                continue 'main;
+                                break;
+                            }
+                            silent_commands(
+                                &mut options,
+                                &s.iter()
+                                    .copied()
+                                    .filter(|&c| !c.is_whitespace())
+                                    .collect::<Vec<char>>(),
+                            );
+                            if s.contains(&'=')
+                            {
+                                if let Err(s) =
+                                    set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
+                                {
+                                    println!("{}", s);
+                                    continue 'main;
+                                }
                             }
                         }
                     }
+                    let tempinput = unparsed.iter().collect::<String>();
+                    if tempinput.starts_with("help ")
+                    {
+                        println!("{}", help_for(tempinput.splitn(2, ' ').last().unwrap()));
+                        continue;
+                    }
+                    else if tempinput.ends_with('=')
+                    {
+                        println!(
+                            "{}",
+                            equal_to(
+                                options,
+                                &colors,
+                                &vars,
+                                &tempinput[..tempinput.len().saturating_sub(1)],
+                                "",
+                            )
+                        );
+                        continue;
+                    }
                 }
-                let tempinput = unparsed.iter().collect::<String>();
-                if tempinput.starts_with("help ")
+                (output, funcvar, graphable, varcheck, _) = match input_var(
+                    &unparsed.iter().map(convert).collect::<String>(),
+                    &vars,
+                    &mut Vec::new(),
+                    &mut 0,
+                    options,
+                    false,
+                    0,
+                    Vec::new(),
+                    false,
+                    &mut Vec::new(),
+                )
                 {
-                    println!("{}", help_for(tempinput.splitn(2, ' ').last().unwrap()));
-                    continue;
-                }
-                else if tempinput.ends_with('=')
-                {
-                    println!(
-                        "{}",
-                        equal_to(
-                            options,
-                            &colors,
-                            &vars,
-                            &tempinput[..tempinput.len().saturating_sub(1)],
-                            "",
-                        )
-                    );
-                    continue;
-                }
-            }
-            (output, funcvar, graphable, varcheck) = match input_var(
-                &unparsed.iter().map(convert).collect::<String>(),
-                &vars,
-                &mut Vec::new(),
-                &mut 0,
-                options,
-                false,
-                0,
-                Vec::new(),
-                false,
-            )
-            {
-                Ok(f) => f,
-                Err(s) =>
-                {
-                    println!("{}: {}", input.iter().collect::<String>(), s);
-                    continue;
-                }
-            };
-            if !graphable.graph && !varcheck
-            {
-                match do_math(output, options, funcvar)
-                {
-                    Ok(n) => print_answer(n, options, &colors),
+                    Ok(f) => f,
                     Err(s) =>
                     {
                         println!("{}: {}", input.iter().collect::<String>(), s);
                         continue;
                     }
-                }
-            }
-            if !graphable.graph && !varcheck
-            {
-                if let Some(time) = watch
+                };
+                if !graphable.graph && !varcheck
                 {
-                    println!(" {}", time.elapsed().as_nanos());
+                    match do_math(output, options, funcvar)
+                    {
+                        Ok(n) => print_answer(n, options, &colors),
+                        Err(s) =>
+                        {
+                            println!("{}: {}", input.iter().collect::<String>(), s);
+                            continue;
+                        }
+                    }
+                    if let Some(time) = watch
+                    {
+                        println!(" {}", time.elapsed().as_nanos());
+                    }
+                    else
+                    {
+                        println!();
+                    }
                 }
-                else
-                {
-                    println!();
-                }
-            }
-            if args.is_empty() && options.stay_interactive
-            {
-                options.interactive = true;
-                terminal::enable_raw_mode().unwrap();
-                print!(
-                    "\x1b[G\x1b[K{}{}",
-                    prompt(options, &colors),
-                    if options.color { "\x1b[0m" } else { "" }
-                );
-                stdout.flush().unwrap();
             }
         }
         else
         {
-            if file.is_none()
+            if !options.interactive
             {
-                for handle in handles
+                if options.stay_interactive
                 {
-                    handle.join().unwrap();
+                    setup_for_interactive(&colors, &mut options, &mut stdout)
+                }
+                else
+                {
+                    for handle in handles
+                    {
+                        handle.join().unwrap();
+                    }
+                    break;
                 }
-                break;
             }
             let mut long = false;
             let mut frac = 0;
@@ -643,9 +667,11 @@ fn main()
             {
                 Vec::new()
             }
-            else if lines[i - 1].starts_with('\0')
+            else if lines[i - 1].ends_with('\t')
             {
-                lines[i - 1][1..].chars().collect::<Vec<char>>()
+                lines[i - 1][..lines[i - 1].len() - 1]
+                    .chars()
+                    .collect::<Vec<char>>()
             }
             else
             {
@@ -681,7 +707,7 @@ fn main()
                             (frac, graphable, _, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -690,32 +716,31 @@ fn main()
                             );
                             slow = watch.elapsed().as_millis() > options.slowcheck;
                         }
-                        if !input.is_empty() && !input.starts_with(&['#'])
-                        {
-                            let num = frac + if !graphable.graph && !varcheck { 1 } else { 0 };
-                            if num != 0
-                            {
-                                print!("\x1b[{}B", num);
-                            }
-                        }
-                        if c == '\x14' || c == '\x06'
-                        {
-                            if input.is_empty()
-                            {
-                                print!("\x1b[G\x1b[J");
-                            }
-                            else
-                            {
-                                print!("\x1b[G\n\x1b[J");
-                            }
-                            terminal::disable_raw_mode().unwrap();
-                            std::process::exit(0);
-                        }
-                        print!("\x1b[G\n\x1b[K");
                         if c == '\x09'
                         {
-                            graphable = HowGraphing::default();
+                            clear(&input, start, end, options, &colors);
                         }
+                        else
+                        {
+                            if !input.is_empty() && !input.starts_with(&['#']) && frac != 0
+                            {
+                                print!("\x1b[{}B", frac);
+                            }
+                            if c == '\x14' || c == '\x06'
+                            {
+                                if input.is_empty()
+                                {
+                                    print!("\x1b[G\x1b[J");
+                                }
+                                else
+                                {
+                                    print!("\x1b[G\n\x1b[J");
+                                }
+                                terminal::disable_raw_mode().unwrap();
+                                std::process::exit(0);
+                            }
+                        }
+                        print!("\x1b[G\n\x1b[K");
                         break;
                     }
                     '\x03' =>
@@ -803,7 +828,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -883,7 +908,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -966,7 +991,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1043,7 +1068,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1088,7 +1113,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1135,7 +1160,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1182,7 +1207,7 @@ fn main()
                                 (frac, graphable, long, varcheck) = print_concurrent(
                                     &input,
                                     &last,
-                                    vars.clone(),
+                                    &vars,
                                     options,
                                     colors.clone(),
                                     start,
@@ -1226,7 +1251,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1289,10 +1314,11 @@ fn main()
                             continue;
                         }
                         input = lines[i].clone().chars().collect::<Vec<char>>();
-                        slow = input.starts_with(&['\0']);
+                        slow = input.ends_with(&['\t']);
+
                         if slow
                         {
-                            input.remove(0);
+                            input.pop();
                             firstslow = false;
                             placement = input.len();
                             end = input.len();
@@ -1340,7 +1366,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1368,10 +1394,10 @@ fn main()
                         {
                             input = lines[i].clone().chars().collect::<Vec<char>>();
                         }
-                        slow = input.starts_with(&['\0']);
+                        slow = input.ends_with(&['\t']);
                         if slow
                         {
-                            input.remove(0);
+                            input.pop();
                             firstslow = false;
                             placement = input.len();
                             end = input.len();
@@ -1419,7 +1445,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1579,6 +1605,224 @@ fn main()
                             }
                         }
                     }
+                    '\x1F' =>
+                    {
+                        //tab completion
+                        let mut word = String::new();
+                        for (i, c) in input[..placement].iter().rev().enumerate()
+                        {
+                            if c.is_alphabetic()
+                                || matches!(*c, '°' | '\'' | '`' | '_' | '∫' | '$' | '¢')
+                            {
+                                word.insert(0, *c)
+                            }
+                            else if !(*c == '(' && i == 0)
+                            {
+                                break;
+                            }
+                        }
+                        if !word.is_empty()
+                        {
+                            let mut bank1 = Vec::new();
+                            for v in &vars
+                            {
+                                let s = v.name.iter().collect::<String>();
+                                if s.starts_with(&word)
+                                {
+                                    bank1.push(s)
+                                }
+                            }
+                            let mut bank2 = Vec::new();
+                            for f in functions_with_args()
+                            {
+                                if f.starts_with(&word)
+                                    && !bank1.iter().any(|b| {
+                                        b.contains('(')
+                                            && b.split('(').next().unwrap()
+                                                == f.split('(').next().unwrap()
+                                    })
+                                {
+                                    bank2.push(f.to_string())
+                                }
+                            }
+                            bank1.extend(bank2);
+                            bank1.sort();
+                            let mut var = false;
+                            if bank1.len() == 1
+                            {
+                                let mut w = bank1[0].to_string();
+                                if w.contains('(')
+                                {
+                                    w = w.split('(').next().unwrap().to_string();
+                                    if (placement == input.len() || input[placement] != '(')
+                                        && input[placement - 1] != '('
+                                    {
+                                        w.push('(')
+                                    }
+                                }
+                                else
+                                {
+                                    var = true
+                                }
+                                let w = w.chars().collect::<Vec<char>>();
+                                input.splice(
+                                    placement..placement,
+                                    w[word.chars().count()..].iter().collect::<String>().chars(),
+                                );
+                                placement += w.len() - word.chars().count();
+                                end = start + get_terminal_dimensions().0
+                                    - if options.prompt { 3 } else { 1 }
+                                    + 1;
+                                if end > input.len()
+                                {
+                                    end = input.len()
+                                }
+                                else if placement == end
+                                {
+                                    start += 1;
+                                }
+                                else
+                                {
+                                    end -= 1;
+                                }
+                                if i == lines.len()
+                                {
+                                    current.clone_from(&input);
+                                }
+                                else
+                                {
+                                    lines[i] = input.clone().iter().collect::<String>();
+                                }
+                                if options.real_time_output && !slow && var
+                                {
+                                    execute!(stdout, DisableBlinking).unwrap();
+                                    (frac, graphable, long, varcheck) = print_concurrent(
+                                        &input,
+                                        &last,
+                                        &vars,
+                                        options,
+                                        colors.clone(),
+                                        start,
+                                        end,
+                                        false,
+                                    );
+                                    if watch.elapsed().as_millis() > options.slowcheck
+                                    {
+                                        firstslow = true;
+                                        slow = true;
+                                    }
+                                }
+                                else if firstslow && var
+                                {
+                                    firstslow = false;
+                                    handle_err(
+                                        "too slow, will print on enter",
+                                        &input,
+                                        options,
+                                        &colors,
+                                        start,
+                                        end,
+                                    )
+                                }
+                                else
+                                {
+                                    clear(&input, start, end, options, &colors);
+                                }
+                                if end - placement != 0
+                                {
+                                    print!("\x1b[{}D", end - placement)
+                                }
+                            }
+                            else if !bank1.is_empty()
+                            {
+                                let mut k = 0;
+                                let mut char = '\0';
+                                'upper: for n in
+                                    0..bank1.iter().fold(usize::MAX, |min, str| min.min(str.len()))
+                                {
+                                    for b in &bank1
+                                    {
+                                        let c = b.chars().nth(n).unwrap();
+                                        if char == '\0'
+                                        {
+                                            char = c
+                                        }
+                                        else if c != char
+                                        {
+                                            break 'upper;
+                                        }
+                                    }
+                                    k += 1;
+                                    char = '\0'
+                                }
+                                input.splice(
+                                    placement..placement,
+                                    bank1[0][word.chars().count()..k].chars(),
+                                );
+                                placement += k - word.chars().count();
+                                end = start + get_terminal_dimensions().0
+                                    - if options.prompt { 3 } else { 1 }
+                                    + 1;
+                                if end > input.len()
+                                {
+                                    end = input.len()
+                                }
+                                else if placement == end
+                                {
+                                    start += 1;
+                                }
+                                else
+                                {
+                                    end -= 1;
+                                }
+                                if i == lines.len()
+                                {
+                                    current.clone_from(&input);
+                                }
+                                else
+                                {
+                                    lines[i] = input.clone().iter().collect::<String>();
+                                }
+                                clear(&input, start, end, options, &colors);
+                                if end - placement != 0
+                                {
+                                    print!("\x1b[{}D", end - placement)
+                                }
+                            }
+                            if !var && !bank1.is_empty()
+                            {
+                                let width = get_terminal_dimensions().0;
+                                let mut n = 1;
+                                let tab = bank1.iter().fold(0, |max, str| max.max(str.len())) + 3;
+                                let mut len = 0;
+                                print!("\x1b[G\n\x1b[K");
+                                for b in bank1
+                                {
+                                    if len + tab > width
+                                    {
+                                        len = 0;
+                                        n += 1;
+                                        print!("\x1b[G\n\x1b[K")
+                                    }
+                                    len += tab;
+                                    print!(
+                                        "{}{}",
+                                        to_output(
+                                            &b.chars().collect::<Vec<char>>(),
+                                            options.color == Auto::True,
+                                            &colors
+                                        ),
+                                        " ".repeat(tab - b.chars().count())
+                                    )
+                                }
+                                print!(
+                                    "\x1b[G\x1b[{}A\x1b[{}C",
+                                    n,
+                                    placement + if options.prompt { 2 } else { 0 }
+                                )
+                            }
+                        }
+                    }
                     '\0' =>
                     {}
                     _ =>
@@ -1614,7 +1858,7 @@ fn main()
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
-                                vars.clone(),
+                                &vars,
                                 options,
                                 colors.clone(),
                                 start,
@@ -1673,7 +1917,14 @@ fn main()
                 print!(
                     "{}{}",
                     prompt(options, &colors),
-                    if options.color { "\x1b[0m" } else { "" }
+                    if options.color == Auto::True
+                    {
+                        "\x1b[0m"
+                    }
+                    else
+                    {
+                        ""
+                    }
                 );
             }
             stdout.flush().unwrap();
@@ -1696,14 +1947,25 @@ fn main()
             {
                 if !s.is_empty()
                 {
-                    print!("\x1b[G\x1b[K{}\x1b[G\n{}", s, prompt(options, &colors));
+                    print!(
+                        "\x1b[G\x1b[A\x1b[K{}\x1b[G\n{}",
+                        s,
+                        prompt(options, &colors)
+                    );
                 }
                 else
                 {
                     print!(
                         "{}{}",
                         prompt(options, &colors),
-                        if options.color { "\x1b[0m" } else { "" }
+                        if options.color == Auto::True
+                        {
+                            "\x1b[0m"
+                        }
+                        else
+                        {
+                            ""
+                        }
                     );
                 }
             }
@@ -1712,7 +1974,14 @@ fn main()
                 print!(
                     "{}{}",
                     prompt(options, &colors),
-                    if options.color { "\x1b[0m" } else { "" }
+                    if options.color == Auto::True
+                    {
+                        "\x1b[0m"
+                    }
+                    else
+                    {
+                        ""
+                    }
                 );
             }
             stdout.flush().unwrap()
@@ -1762,3 +2031,21 @@ fn main()
         }
     }
 }
+fn setup_for_interactive(colors: &Colors, options: &mut Options, stdout: &mut Stdout)
+{
+    options.interactive = true;
+    terminal::enable_raw_mode().unwrap();
+    print!(
+        "\x1b[G\x1b[K{}{}",
+        prompt(*options, colors),
+        if options.color == Auto::True
+        {
+            &colors.text
+        }
+        else
+        {
+            ""
+        }
+    );
+    stdout.flush().unwrap();
+}
