@@ -210,6 +210,7 @@ pub fn read_single_char() -> char
                     (KeyCode::Right, KeyModifiers::NONE) => '\x1C',
                     (KeyCode::Up, KeyModifiers::NONE) => '\x1D',
                     (KeyCode::Down, KeyModifiers::NONE) => '\x1E',
+                    (KeyCode::Tab, KeyModifiers::NONE) => '\x1F',
                     _ => '\0',
                 }
             }
@@ -252,13 +253,44 @@ pub fn no_col(input: &str, color: bool) -> Vec<char>
         input.chars().collect::<Vec<char>>()
     }
 }
+pub fn no_col_len(input: &str, color: bool) -> usize
+{
+    if color
+    {
+        let mut skip = false;
+        let mut count = 0;
+        for c in input.chars()
+        {
+            if skip
+            {
+                if c == 'm' || c == 'G' || c == 'K'
+                {
+                    skip = false
+                }
+            }
+            else if c == '\x1b'
+            {
+                skip = true
+            }
+            else
+            {
+                count += 1
+            }
+        }
+        count
+    }
+    else
+    {
+        input.len()
+    }
+}
 pub fn write(mut input: String, file: &mut File, lines: &mut Vec<String>, slow: bool, last: String)
 {
     if last != input && !input.replace(' ', "").is_empty() && !input.starts_with('#')
     {
         if slow
         {
-            input.insert(0, '\0');
+            input.push('\t');
         }
         lines.push(input.clone());
         writeln!(file, "{}", input).unwrap();
@@ -269,8 +301,19 @@ pub fn clearln(input: &[char], start: usize, end: usize, options: Options, color
     print!(
         "\x1b[G{}{}\x1b[K{}",
         prompt(options, colors),
-        to_output(&input[start..end], options.color, colors),
-        if options.color { "\x1b[0m" } else { "" }
+        to_output(
+            &input[start..end],
+            options.color == crate::Auto::True,
+            colors
+        ),
+        if options.color == crate::Auto::True
+        {
+            "\x1b[0m"
+        }
+        else
+        {
+            ""
+        }
     );
 }
 pub fn clear(input: &[char], start: usize, end: usize, options: Options, colors: &Colors)
@@ -278,8 +321,19 @@ pub fn clear(input: &[char], start: usize, end: usize, options: Options, colors:
     print!(
         "\x1b[G{}{}\x1b[J{}",
         prompt(options, colors),
-        to_output(&input[start..end], options.color, colors),
-        if options.color { "\x1b[0m" } else { "" }
+        to_output(
+            &input[start..end],
+            options.color == crate::Auto::True,
+            colors
+        ),
+        if options.color == crate::Auto::True
+        {
+            "\x1b[0m"
+        }
+        else
+        {
+            ""
+        }
     );
 }
 pub fn to_output(input: &[char], color: bool, colors: &Colors) -> String
@@ -397,8 +451,19 @@ pub fn handle_err(
             format!("\x1b[{}A", num)
         },
         prompt(options, colors),
-        to_output(&input[start..end], options.color, colors),
-        if options.color { "\x1b[0m" } else { "" },
+        to_output(
+            &input[start..end],
+            options.color == crate::Auto::True,
+            colors
+        ),
+        if options.color == crate::Auto::True
+        {
+            "\x1b[0m"
+        }
+        else
+        {
+            ""
+        },
     );
 }
 pub fn prompt(options: Options, colors: &Colors) -> String
@@ -409,7 +474,7 @@ pub fn prompt(options: Options, colors: &Colors) -> String
     }
     else if options.prompt
     {
-        if options.color
+        if options.color == crate::Auto::True
         {
             format!("{}>{} ", colors.prompt, colors.text)
         }
@@ -418,7 +483,7 @@ pub fn prompt(options: Options, colors: &Colors) -> String
             "> ".to_string()
         }
     }
-    else if options.color
+    else if options.color == crate::Auto::True
     {
         colors.text.to_string()
     }
@@ -455,17 +520,20 @@ pub fn place_funcvar(
     num: NumStr,
 ) -> Vec<(String, Vec<NumStr>)>
 {
-    for i in funcvar.iter_mut()
+    if !var.is_empty()
     {
-        if !i.0.contains('(')
+        for i in funcvar.iter_mut()
         {
-            for j in i.1.iter_mut()
+            if !i.0.contains('(')
             {
-                match j
+                for j in i.1.iter_mut()
                 {
-                    Str(s) if s == var => *j = num.clone(),
-                    _ =>
-                    {}
+                    match j
+                    {
+                        Str(s) if s == var => *j = num.clone(),
+                        _ =>
+                        {}
+                    }
                 }
             }
         }
@@ -497,7 +565,7 @@ pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
                     }
                     "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
                     | "integrate" | "arclength" | "∫" | "area" | "surfacearea" | "sarea"
-                    | "solve" | "length" | "slope" | "lim" | "limit" | "iter"
+                    | "solve" | "length" | "slope" | "lim" | "limit" | "iter" | "extrema"
                         if i + 2 < func.len()
                             && if let Str(s) = &func[i + 2]
                             {
@@ -536,47 +604,51 @@ pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
 }
 pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
 {
-    let mut sum = Vec::new();
-    let mut bracket = 0;
-    let mut i = 0;
-    while func.len() > i
+    if !var.is_empty()
     {
-        if let Str(s) = &func[i]
+        let mut sum = Vec::new();
+        let mut bracket = 0;
+        let mut i = 0;
+        while func.len() > i
         {
-            if s == var && sum.is_empty()
+            if let Str(s) = &func[i]
             {
-                func[i] = num.clone();
-            }
-            else
-            {
-                match s.as_str()
+                if s == var && sum.is_empty()
                 {
-                    "(" => bracket += 1,
-                    ")" => bracket -= 1,
-                    "," if sum.contains(&bracket) =>
+                    func[i] = num.clone();
+                }
+                else
+                {
+                    match s.as_str()
                     {
-                        sum.remove(0);
+                        "(" => bracket += 1,
+                        ")" => bracket -= 1,
+                        "," if sum.contains(&bracket) =>
+                        {
+                            sum.remove(0);
+                        }
+                        "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat"
+                        | "D" | "integrate" | "arclength" | "∫" | "area" | "solve" | "length"
+                        | "slope" | "lim" | "limit" | "iter" | "extrema" | "surfacearea"
+                        | "sarea"
+                            if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
+                        {
+                            i += 3;
+                            sum.push(bracket)
+                        }
+                        "surfacearea" | "sarea"
+                            if i + 4 < func.len() && func[i + 4] == Str(var.to_string()) =>
+                        {
+                            i += 5;
+                            sum.push(bracket)
+                        }
+                        _ =>
+                        {}
                     }
-                    "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
-                    | "integrate" | "arclength" | "∫" | "area" | "solve" | "length" | "slope"
-                    | "lim" | "limit" | "iter" | "surfacearea" | "sarea"
-                        if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
-                    {
-                        i += 3;
-                        sum.push(bracket)
-                    }
-                    "surfacearea" | "sarea"
-                        if i + 4 < func.len() && func[i + 4] == Str(var.to_string()) =>
-                    {
-                        i += 5;
-                        sum.push(bracket)
-                    }
-                    _ =>
-                    {}
                 }
             }
+            i += 1;
         }
-        i += 1;
     }
     func
 }
@@ -632,7 +704,14 @@ pub fn parsed_to_string(
                     n.0,
                     n.1,
                     n.2.unwrap_or_default(),
-                    if options.color { "\x1b[0m" } else { "" }
+                    if options.color == crate::Auto::True
+                    {
+                        "\x1b[0m"
+                    }
+                    else
+                    {
+                        ""
+                    }
                 )
             }
             Vector(n) =>
@@ -647,7 +726,14 @@ pub fn parsed_to_string(
                         num.0,
                         num.1,
                         num.2.unwrap_or_default(),
-                        if options.color { "\x1b[0m" } else { "" }
+                        if options.color == crate::Auto::True
+                        {
+                            "\x1b[0m"
+                        }
+                        else
+                        {
+                            ""
+                        }
                     ));
                 }
                 str.pop();
@@ -668,7 +754,14 @@ pub fn parsed_to_string(
                             num.0,
                             num.1,
                             num.2.unwrap_or_default(),
-                            if options.color { "\x1b[0m" } else { "" }
+                            if options.color == crate::Auto::True
+                            {
+                                "\x1b[0m"
+                            }
+                            else
+                            {
+                                ""
+                            }
                         ));
                     }
                     str.insert(str.len().saturating_sub(1), '}');
@@ -683,7 +776,11 @@ pub fn parsed_to_string(
             Str(n) => n.replace('@', ""),
         })
     }
-    to_output(&out.chars().collect::<Vec<char>>(), options.color, colors)
+    to_output(
+        &out.chars().collect::<Vec<char>>(),
+        options.color == crate::Auto::True,
+        colors,
+    )
 }
 pub fn insert_last(input: &[char], last: &str) -> String
 {
